==> racer-0.1.1.tgz: package/lib/mixin.stm/Async.js <==
var Async, AsyncAtomic, MAX_RETRIES, RETRY_DELAY, empty, transaction;
transaction = require('../transaction');
MAX_RETRIES = 20;
RETRY_DELAY = 5;
AsyncAtomic = function(async, cb) {
  this.async = async;
  this.cb = cb;
  this.minVer = 0;
  this.count = 0;
};
AsyncAtomic.prototype = {
  _reset: function() {
    this.minVer = 0;
    return this.count = 0;
  },
  get: function(path, callback) {
    var cb, minVer, self;
    minVer = this.minVer;
    cb = this.cb;
    self = this;
    return this.async.get(path, function(err, value, ver) {
      if (err) {
        return cb(err);
      }
      self.minVer = minVer ? Math.min(minVer, ver) : ver;
      if (callback) {
        return callback(value);
      }
    });
  },
  set: function(path, value, callback) {
    var cb, self;
    this.count++;
    cb = this.cb;
    self = this;
    return this.async.set(path, value, this.minVer, function(err, value) {
      if (err) {
        return cb(err);
      }
      if (callback) {
        callback(null, value);
      }
      if (!--self.count) {
        return cb();
      }
    });
  },
  del: function(path, callback) {
    var cb, self;
    this.count++;
    cb = this.cb;
    self = this;
    return this.async.del(path, this.minVer, function(err) {
      if (err) {
        return cb(err);
      }
      if (callback) {
        callback();
      }
      if (!--self.count) {
        return cb();
      }
    });
  }
};
empty = function() {};
module.exports = Async = function(model) {
  this.model = model;
};
Async.prototype = {
  _nextTxnId: function() {
    return '#' + this.model._nextTxnId();
  },
  get: function(path, callback) {
    return this.model.store._adapter.get(path, callback);
  },
  set: function(path, value, ver, callback) {
    var txn;
    txn = transaction.create({
      base: ver,
      id: this._nextTxnId(),
      method: 'set',
      args: [path, value]
    });
    return this.model.store._commit(txn, callback);
  },
  del: function(path, ver, callback) {
    var txn;
    txn = transaction.create({
      base: ver,
      id: this._nextTxnId(),
      method: 'del',
      args: [path]
    });
    return this.model.store._commit(txn, callback);
  },
  incr: function(path, byNum, callback) {
    var tryVal;
    if (typeof byNum === 'function') {
      callback = byNum;
      byNum = 1;
    } else {
      if (byNum == null) {
        byNum = 1;
      }
      callback || (callback = empty);
    }
    tryVal = null;
    return this.retry(function(atomic) {
      return atomic.get(path, function(val) {
        return atomic.set(path, tryVal = (val || 0) + byNum);
      });
    }, function(err) {
      return callback(err, tryVal);
    });
  },
  retry: function(fn, callback) {
    var atomic, retries;
    retries = MAX_RETRIES;
    atomic = new AsyncAtomic(this, function(err) {
      var delay;
      if (!err) {
        return callback && callback();
      }
      if (!retries--) {
        return callback && callback('maxRetries');
      }
      atomic._reset();
      delay = (MAX_RETRIES - retries) * RETRY_DELAY;
      return setTimeout(fn, delay, atomic);
    });
    return fn(atomic);
  }
};

==> racer-0.1.3.tgz: package/lib/mixin.stm/Async.js <==
var Async, AsyncAtomic, MAX_RETRIES, RETRY_DELAY, empty, transaction;

transaction = require('../transaction');

MAX_RETRIES = 20;

RETRY_DELAY = 5;

AsyncAtomic = function(async, cb) {
  this.async = async;
  this.cb = cb;
  this.minVer = 0;
  this.count = 0;
};

AsyncAtomic.prototype = {
  _reset: function() {
    this.minVer = 0;
    return this.count = 0;
  },
  get: function(path, callback) {
    var cb, minVer, self;
    minVer = this.minVer;
    cb = this.cb;
    self = this;
    return this.async.get(path, function(err, value, ver) {
      if (err) return cb(err);
      self.minVer = minVer ? Math.min(minVer, ver) : ver;
      if (callback) return callback(value);
    });
  },
  set: function(path, value, callback) {
    var cb, self;
    this.count++;
    cb = this.cb;
    self = this;
    return this.async.set(path, value, this.minVer, function(err, value) {
      if (err) return cb(err);
      if (callback) callback(null, value);
      if (!--self.count) return cb();
    });
  },
  del: function(path, callback) {
    var cb, self;
    this.count++;
    cb = this.cb;
    self = this;
    return this.async.del(path, this.minVer, function(err) {
      if (err) return cb(err);
      if (callback) callback();
      if (!--self.count) return cb();
    });
  }
};

empty = function() {};

module.exports = Async = function(model) {
  this.model = model;
};

Async.prototype = {
  _nextTxnId: function() {
    return '#' + this.model._nextTxnId();
  },
  get: function(path, callback) {
    return this.model.store._adapter.get(path, callback);
  },
  set: function(path, value, ver, callback) {
    var txn;
    txn = transaction.create({
      base: ver,
      id: this._nextTxnId(),
      method: 'set',
      args: [path, value]
    });
    return this.model.store._commit(txn, callback);
  },
  del: function(path, ver, callback) {
    var txn;
    txn = transaction.create({
      base: ver,
      id: this._nextTxnId(),
      method: 'del',
      args: [path]
    });
    return this.model.store._commit(txn, callback);
  },
  incr: function(path, byNum, callback) {
    var tryVal;
    if (typeof byNum === 'function') {
      callback = byNum;
      byNum = 1;
    } else {
      if (byNum == null) byNum = 1;
      callback || (callback = empty);
    }
    tryVal = null;
    return this.retry(function(atomic) {
      return atomic.get(path, function(val) {
        return atomic.set(path, tryVal = (val || 0) + byNum);
      });
    }, function(err) {
      return callback(err, tryVal);
    });
  },
  retry: function(fn, callback) {
    var atomic, retries;
    retries = MAX_RETRIES;
    atomic = new AsyncAtomic(this, function(err) {
      var delay;
      if (!err) return callback && callback();
      if (!retries--) return callback && callback('maxRetries');
      atomic._reset();
      delay = (MAX_RETRIES - retries) * RETRY_DELAY;
      return setTimeout(fn, delay, atomic);
    });
    return fn(atomic);
  }
};
